#include <unistd.h>
#include <signal.h>
#include <time.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#if defined __i386__
#include <asm/msr.h>
#endif
#include "posix_time.h"
#include "clock_linux_counter.h"

static long hz;

#define READ_LABEL   0
#define SKIP_TO_EOL  1
#define READ_VALUE   2

static int get_hz(void)
{
#if 0
  errno = 0;
  hz = sysconf(_SC_LINUX_COUNTER_HZ);
#else
  FILE *cpu_f;
  char buf[512];

  hz = -1;
  cpu_f = fopen("/proc/cpuinfo", "r");
  if (cpu_f == NULL) return -1;
  while (fgets(buf, sizeof(buf), cpu_f)) {
    if (strncmp(buf, "cpu MHz", 7) == 0) {
      char *colon;

      colon = strchr(buf, ':');
      if (colon == NULL) break;

      hz = atof(colon + 1) * 1e6;
    }
  }
  fclose(cpu_f);
#endif
  return hz;
}

static int linux_counter_clock_gettime(clockid_t which_clock,
				       struct timespec *ts)
{
  long long count;

  (void)which_clock;

  if ((hz == 0) && (get_hz() == -1)) return -1;

  rdtscll(count);

  ts->tv_sec = count / hz;
  ts->tv_nsec = (count - (long long)ts->tv_sec * hz) * NSEC_PER_SEC / hz;

  return 0;
}

static int linux_counter_clock_getres(clockid_t which_clock,
				      struct timespec *resolution)
{
  (void)which_clock;

  if ((hz == 0) && (get_hz() == -1)) return -1;

  resolution->tv_sec = 0;
  resolution->tv_nsec = NSEC_PER_SEC/hz;

  return 0;
}

struct clock_func clock_linux_counter =
{
  linux_counter_clock_gettime,
  NULL,
  linux_counter_clock_getres,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL
};
