/*
 * Copyright (C) 1997 by the University of Kansas Center for Research,
 * Inc.  This software was developed by the Information and
 * Telecommunication Technology Center (ITTC) at the University of
 * Kansas.  Partial funding for this project was provided by Sprint. This
 * software may be used and distributed according to the terms of the GNU
 * Public License, incorporated herein by reference.  Neither ITTC nor
 * Sprint accept any liability whatsoever for this product.
 *
 * This project was developed under the direction of Dr. Douglas Niehaus.
 *
 * Authors: Shyam Pather, Balaji Srinivasan 
 *
 * Please send bug-reports/suggestions/comments to posix@ittc.ukans.edu
 *
 * Further details about this project can be obtained at
 *    http://hegel.ittc.ukans.edu/projects/posix/
 */

#ifndef __POSIX_TIME_H
#define __POSIX_TIME_H

#include <signal.h>
#include <time.h>

#ifdef _POSIX_TIMERS

#define CLOCK_REALTIME		0
#define CLOCK_LINUX_COUNTER	1
#define CLOCK_USER1		8
#define CLOCK_USER2		9

#define NOF_CLOCKS 10

#define TIMER_ABSTIME 0x01

#define TIMER_MAX 32

#define NSEC_PER_SEC 1000000000L

typedef int clockid_t;
typedef int timer_t;

struct  itimerspec {
        struct  timespec it_interval;    /* timer period */
        struct  timespec it_value;       /* timer expiration */
};

struct clock_func {
  int (*clock_gettime)(clockid_t, struct timespec*);
  int (*clock_settime)(clockid_t, const struct timespec*);
  int (*clock_getres)(clockid_t, struct timespec*);
  int (*timer_create)(clockid_t, struct sigevent*, timer_t*);
  int (*timer_gettime)(timer_t, struct itimerspec*);
  int (*timer_settime)(timer_t, int flags, const struct itimerspec*,
		       struct itimerspec*);
  int (*timer_getoverrun)(timer_t);
  int (*timer_delete)(timer_t);
};

#ifdef __cplusplus
extern "C" {
#endif

int timer_create(clockid_t which_clock, struct sigevent *timer_event_spec,
                 timer_t *created_timer_id);

int timer_gettime(timer_t timer_id, struct itimerspec *setting);

int timer_settime(timer_t timer_id,
                  int flags,
                  const struct itimerspec *new_setting,
                  struct itimerspec *old_setting);

int timer_getoverrun(timer_t timer_id);

int timer_delete(timer_t timer_id);

int clock_gettime(clockid_t which_clock, struct timespec *ts);

int clock_settime(clockid_t which_clock, const struct timespec *setting);

int clock_getres(clockid_t which_clock, struct timespec *resolution);

int clock_install(clockid_t which_clock, struct clock_func *new_clock,
		  struct clock_func **old_clock);

#ifdef __cplusplus
}
#endif

#endif /* POSIX_TIMERS */

#endif /* __POSIX_TIME_H */
